@@ -2,9 +2,9 @@ use warnings;
 use strict;
 
 package App::ClusterSSH::Base;
-$App::ClusterSSH::Base::VERSION = '4.13.2_02'; # TRIAL
-
-$App::ClusterSSH::Base::VERSION = '4.13.202';# ABSTRACT: App::ClusterSSH::Base - Base object provding utility functions
+$App::ClusterSSH::Base::VERSION = '4.18_01'; # TRIAL
+$App::ClusterSSH::Base::VERSION = '4.1801';
+# ABSTRACT: App::ClusterSSH::Base - Base object provding utility functions
 
 =head1 SYNOPSIS
 
@@ -305,15 +305,13 @@ sub parent {
 sub sort {
     my $self = shift;
 
-    my $sort = sub { sort @_ };
-
-    return $sort
-        unless ref( $self->config() ) eq "HASH"
-        && $self->config()->{'use_natural_sort'};
-
     # if the user has asked for natural sorting we need to include an extra
     # module
-    if ( $self->config()->{'use_natural_sort'} ) {
+    my $config = $self->config();
+
+    # Make sure the configuration object has been set correctly before
+    # referencing anything
+    if ( ref $config eq "HASH" && $config->{'use_natural_sort'} ) {
         eval { Module::Load::load('Sort::Naturally'); };
         if ($@) {
             warn(
@@ -321,10 +319,12 @@ sub sort {
             );
         }
         else {
-            $sort = sub { Sort::Naturally::nsort(@_) };
+            my $sort = sub { Sort::Naturally::nsort(@_) };
+            return $sort;
         }
     }
 
+    my $sort = sub { sort @_ };
     return $sort;
 }
 
