@@ -2,9 +2,9 @@ use strict;
 use warnings;
 
 package App::ClusterSSH::Config;
-$App::ClusterSSH::Config::VERSION = '4.13.2_02'; # TRIAL
-
-$App::ClusterSSH::Config::VERSION = '4.13.202';# ABSTRACT: ClusterSSH::Config - Object representing application configuration
+$App::ClusterSSH::Config::VERSION = '4.18_01'; # TRIAL
+$App::ClusterSSH::Config::VERSION = '4.1801';
+# ABSTRACT: ClusterSSH::Config - Object representing application configuration
 
 =head1 SYNOPSIS
 
@@ -50,6 +50,10 @@ my %default_config = (
     key_macros_enable       => "Alt-p",
     key_paste               => "Control-v",
     key_username            => "Alt-u",
+    key_user_1              => "Alt-1",
+    key_user_2              => "Alt-2",
+    key_user_3              => "Alt-3",
+    key_user_4              => "Alt-4",
     mouse_paste             => "Button-2",
     auto_quit               => "yes",
     auto_close              => 5,
@@ -95,6 +99,8 @@ my %default_config = (
     history_height => 10,
 
     command             => q{},
+    command_pre         => q{},
+    command_post        => q{},
     hide_menu           => 0,
     max_host_menu_items => 30,
 
@@ -104,6 +110,16 @@ my %default_config = (
     macro_username   => '%u',
     macro_newline    => '%n',
     macro_version    => '%v',
+    macro_user_1     => '%1',
+    macro_user_2     => '%2',
+    macro_user_3     => '%3',
+    macro_user_4     => '%4',
+
+    macro_user_1_command => '',
+    macro_user_2_command => '',
+    macro_user_3_command => '',
+    macro_user_4_command => '',
+    hostname_override    => '',
 
     max_addhost_menu_cluster_items => 6,
     menu_send_autotearoff          => 0,
@@ -280,6 +296,15 @@ sub parse_config_file {
         if ( $read_config{terminal_font} );
 
     $self->validate_args(%read_config);
+
+    # Look at the user macros and if not set remove the hotkey for them
+    for my $i (qw/ 1 2 3 4 /) {
+        if ( !$self->{"macro_user_${i}_command"} ) {
+            delete $self->{"key_user_${i}"};
+        }
+    }
+
+    return $self;
 }
 
 sub load_configs {
@@ -291,7 +316,7 @@ sub load_configs {
         $ENV{HOME} . '/.clusterssh/config',
         )
     {
-        $self->parse_config_file($config) if ( -e $config );
+        $self->parse_config_file($config) if ( -e $config && !-d _ );
     }
 
     # write out default config file if necesasry
@@ -306,10 +331,22 @@ sub load_configs {
     # relative to config directory
     for my $config (@configs) {
         next unless ($config);    # can be null when passed from Getopt::Long
-        $self->parse_config_file($config) if ( -e $config );
+        $self->parse_config_file($config) if ( -e $config && !-d _ );
 
         my $file = $ENV{HOME} . '/.clusterssh/config_' . $config;
-        $self->parse_config_file($file) if ( -e $file );
+        $self->parse_config_file($file) if ( -e $file && !-d _ );
+    }
+
+    # Override confuration via environment variable using cssh_ prefix
+    #  eg: terminal_size => cssh_terminal_size
+    foreach my $config_key ( sort( keys(%default_config) ) ) {
+        my $env_config_key = "cssh_" . $config_key;
+        if ( exists $ENV{ uc($env_config_key) } ) {
+            $env_config_key = uc($env_config_key);
+        }
+        if ( exists $ENV{$env_config_key} ) {
+            $self->{$config_key} = $ENV{$env_config_key};
+        }
     }
 
     return $self;
