@@ -2,9 +2,9 @@ use strict;
 use warnings;
 
 package App::ClusterSSH::Getopt;
-$App::ClusterSSH::Getopt::VERSION = '4.13.2_02'; # TRIAL
-
-$App::ClusterSSH::Getopt::VERSION = '4.13.202';# ABSTRACT: App::ClusterSSH::Getopt - module to process command line args
+$App::ClusterSSH::Getopt::VERSION = '4.18_01'; # TRIAL
+$App::ClusterSSH::Getopt::VERSION = '4.1801';
+# ABSTRACT: App::ClusterSSH::Getopt - module to process command line args
 
 =head1 SYNOPSIS
 
@@ -96,7 +96,7 @@ sub add_option {
         }
         $desc  .= " $arg" if ($arg);
         $short .= " $arg" if ( $short && $arg );
-        $long  .= " $arg" if ( $long && $arg );
+        $long  .= " $arg" if ( $long  && $arg );
     }
     $args{option_desc}  = $desc;
     $args{option_short} = $short;
@@ -150,10 +150,16 @@ sub add_common_options {
             'Number of seconds to wait before closing finished terminal windows.'
         ),
     );
+    $self->add_option(
+        spec => 'no-autoclose',
+        help => $self->loc(
+            'Do not close terminal windows automatically (same as auto_close = 0).'
+        ),
+    );
     $self->add_option(
         spec => 'autoquit|q',
         help => $self->loc(
-            'Toggle automatically quiting after the last client window has closed (overriding the config file).'
+            'Toggle automatically quitting after the last client window has closed (overriding the config file).'
         ),
     );
     $self->add_option(
@@ -394,6 +400,8 @@ sub getopts {
     $self->parent->config->{auto_close} = $self->autoclose
         if ( $self->autoclose );
 
+    $self->parent->config->{auto_close} = 0 if ( $self->no_autoclose );
+
     if ( $self->autoquit ) {
         $self->parent->config->{auto_quit}
             = !$self->parent->config->{auto_quit} || 0;
@@ -427,7 +435,7 @@ sub output {
 sub _generate_pod {
     my ($self) = @_;
 
-    output $/ , "=pod";
+    output $/,           "=pod";
     output '=head1 ',    $self->loc('NAME');
     output "$Script - ", $self->loc("Cluster administration tool");
     output '=head1 ',    $self->loc('VERSION');
@@ -447,6 +455,28 @@ sub _generate_pod {
     }
     print $/, $/;
 
+    output '=head1 ', $self->loc('RELATED');
+    output $self->loc(
+        q{Also see the individual man pages for each of these utilities});
+
+    my %utils = (
+        ctel  => 'telnet',
+        cssh  => 'ssh',
+        crsh  => 'rsh',
+        csftp => 'sftp',
+        ccon  => 'console',
+    );
+
+    output '=over';
+
+    for my $util ( sort grep { !/$Script/ } keys %utils ) {
+        output "=item $util - "
+            . $self->loc( q{Use '[_1]' as the communication method},
+            $utils{$util} );
+    }
+
+    output '=back';
+
     output '=head1 ', $self->loc('DESCRIPTION');
     output $self->loc(
         q{The command opens an administration console and an xterm to all specified hosts.  Any text typed into the administration console is replicated to all windows.  All windows may also be typed into directly.
@@ -560,6 +590,14 @@ would replace the <Alt-n> with the client's name in each window.}
     output $self->loc(q{Retile all the client windows.});
     output '=item ', $self->parent->config->{key_username};
     output $self->loc(q{Paste in the username for the connection});
+    output '=item ', $self->parent->config->{key_user_1} || 'Alt-1';
+    output '=item ', $self->parent->config->{key_user_2} || 'Alt-2';
+    output '=item ', $self->parent->config->{key_user_3} || 'Alt-3';
+    output '=item ', $self->parent->config->{key_user_4} || 'Alt-4';
+    output $self->loc(
+        q{Run the matching user defined macro on the server and send the output to the client}
+    );
+
     output '=back';
 
     output '=head1 ' . $self->loc('EXAMPLES');
@@ -673,6 +711,18 @@ would replace the <Alt-n> with the client's name in each window.}
         'Enable or disable alternative algorithm for calculating terminal positioning.',
     );
 
+    output '=item command_pre =';
+    output '=item command_post =';
+    output $self->loc(
+        q{Add extra commands around the communication method.  For example:
+
+command_pre= . $HOME/virtualenvs/default/bin/active ;
+command_post= | ct
+
+would allow for using Python virtual envronments and then piping all shell output through C<chromaterm> for syntax highlighting.  Note: you must use appropriate command separators/terminators to keep the meaning of the command pipline (such as C<;> and C<|> between commands).
+
+These are not put through macro parsing.}
+    );
     output '=item comms = ' . $self->parent->config->{comms};
     output $self->loc(
         'Sets the default communication method (initially taken from the name of the program, but can be overridden here).'
@@ -695,7 +745,7 @@ Commands:
 C<< open <tag|hostname> >> - open new sessions to provided tag or hostname
 
 C<< retile >> - force window retiling
-    
+
 e.g.: C<< echo 'open localhost' >> /path/to/external_command_pipe >>}
     );
 
@@ -769,15 +819,64 @@ If the external command is given a C<-L> option it should output a list of tags
         'L<KEY SHORTCUTS>'
     );
 
+    output '=item key_user_1 = Alt-1';
+    output '=item key_user_2 = Alt-2';
+    output '=item key_user_3 = Alt-3';
+    output '=item key_user_4 = Alt-4';
+    output $self->loc(
+        q{Default key sequence to send user defined macros to client.  If the matching [_2] macro is undefined, the sequence is passed straight to the terminal.  See [_1] for more information.},
+        'L<KEY SHORTCUTS>',
+        'L<macro_user_1>'
+    );
+
     output '=item macro_servername = %s';
     output '=item macro_hostname = %h';
     output '=item macro_username = %u';
     output '=item macro_newline = %n';
     output '=item macro_version = %v';
+    output '=item macro_user_1 = %1';
+    output '=item macro_user_2 = %2';
+    output '=item macro_user_3 = %3';
+    output '=item macro_user_4 = %4';
     output $self->loc(
         q{Change the replacement macro used when either using a 'Send' menu item, or when pasting text into the main console.}
     );
 
+    output '=item macro_user_1_command =';
+    output '=item macro_user_2_command =';
+    output '=item macro_user_3_command =';
+    output '=item macro_user_4_command =';
+
+    output $self->loc(
+        q{User defined macros - the macro is run through the shell on the server and the output is sent to the client.  For example,},
+    );
+
+    output "C<macro_user_1_command=echo echo macro_user_1>";
+    output $self->loc(
+        q{
+            would send the text [_1] into the terminal session.
+        },
+        'C<echo macro_user_1>'
+    );
+    output "C<macro_user_1_command=env | grep CSSH>";
+    output $self->loc(
+        q{
+            would send the CSSH environment variables to the client.
+        },
+    );
+
+    output $self->loc(
+        "The following environment variables are set in the shell of the macro process"
+    );
+    output '=over';
+    output '=item C<CSSH_SERVERNAME>';
+    output '=item C<CSSH_HOSTNAME>';
+    output '=item C<CSSH_USERNAME>';
+    output '=item C<CSSH_CONNECTION_STRING>';
+    output '=item C<CSSH_CONNECTION_PORT>';
+    output '=item C<CSSH_VERSION>';
+    output '=back';
+
     output '=item macros_enabled = yes';
     output $self->loc(
         q{Enable or disable macro replacement.  Note: this affects all the [_1] variables above.},
@@ -1128,17 +1227,17 @@ Add extra options into the allowed set for parsing from the command line
 
 =item $obj=ClusterSSH::Getopts->add_common_options ({ })
 
-Add common options used by most calling scripts into the allowed set for 
+Add common options used by most calling scripts into the allowed set for
 parsing from the command line
 
 =item $obj=ClusterSSH::Getopts->add_common_session_options ({ })
 
-Add common session options used by most calling scripts into the allowed 
+Add common session options used by most calling scripts into the allowed
 set for parsing from the command line
 
 =item $obj=ClusterSSH::Getopts->add_common_ssh_options ({ })
 
-Add common ssh options used by most calling scripts into the allowed 
+Add common ssh options used by most calling scripts into the allowed
 set for parsing from the command line
 
 =item $obj->getopts
