@@ -2,9 +2,9 @@ use strict;
 use warnings;
 
 package App::ClusterSSH::Helper;
-$App::ClusterSSH::Helper::VERSION = '4.13.2_02'; # TRIAL
-
-$App::ClusterSSH::Helper::VERSION = '4.13.202';# ABSTRACT: ClusterSSH::Helper - Object representing helper script
+$App::ClusterSSH::Helper::VERSION = '4.18_01'; # TRIAL
+$App::ClusterSSH::Helper::VERSION = '4.1801';
+# ABSTRACT: ClusterSSH::Helper - Object representing helper script
 
 =head1 SYNOPSIS
 
@@ -53,6 +53,8 @@ sub script {
         }
     }
 
+    my $command_pre    = $config->{command_pre}  || q{};
+    my $command_post   = $config->{command_post} || q{};
     my $comms          = $config->{ $config->{comms} };
     my $comms_args     = $config->{ $config->{comms} . '_args' };
     my $config_command = $config->{command};
@@ -70,9 +72,9 @@ sub script {
            my \$user=shift;
            my \$port=shift;
            my \$mstr=shift;
-           my \$command="$comms $comms_args ";
+           my \$command="$command_pre $comms $comms_args ";
            open(PIPE, ">", \$pipe) or die("Failed to open pipe: \$!\\n");
-           print PIPE "\$\$:\$ENV{WINDOWID}" 
+           print PIPE "\$\$:\$ENV{WINDOWID}"
                or die("Failed to write to pipe: $!\\n");
            close(PIPE) or die("Failed to close pipe: $!\\n");
            if(\$svr =~ m/==\$/)
@@ -106,6 +108,7 @@ sub script {
            if("$config_command") {
             \$command .= " \\\"$config_command\\\"";
            }
+           \$command .= "$command_post";
            \$command .= " ; $postcommand";
            # provide some info for debugging purposes
            warn("Running: \$command\\n");
