@@ -2,16 +2,17 @@ use strict;
 use warnings;
 
 package App::ClusterSSH::Window::Tk;
-$App::ClusterSSH::Window::Tk::VERSION = '4.13.2_02'; # TRIAL
-
-$App::ClusterSSH::Window::Tk::VERSION = '4.13.202';# ABSTRACT: Object for creating windows using Tk
+$App::ClusterSSH::Window::Tk::VERSION = '4.18_01'; # TRIAL
+$App::ClusterSSH::Window::Tk::VERSION = '4.1801';
+# ABSTRACT: Object for creating windows using Tk
 
 use English qw( -no_match_vars );
 
 use base qw/ App::ClusterSSH::Base /;
 use vars qw/ %keysymtocode %keycodetosym /;
 
-use File::Temp qw/:POSIX/;
+use Sys::Hostname qw/ hostname /;
+use File::Temp    qw/:POSIX/;
 use Fcntl;
 use POSIX ":sys_wait_h";
 use POSIX qw/:sys_wait_h strftime mkfifo/;
@@ -20,19 +21,21 @@ use Tk::Xlib;
 use Tk::ROText;
 require Tk::Dialog;
 require Tk::LabEntry;
+use Symbol qw/ gensym /;
+use IPC::Open3;
 use X11::Protocol 0.56;
 use X11::Protocol::Constants qw/ Shift Mod5 ShiftMask /;
 use X11::Protocol::WM 29;
 use X11::Keysyms '%keysymtocode', 'MISCELLANY', 'XKB_KEYS', '3270', 'LATIN1',
     'LATIN2', 'LATIN3', 'LATIN4', 'KATAKANA', 'ARABIC', 'CYRILLIC', 'GREEK',
-    'TECHNICAL', 'SPECIAL', 'PUBLISHING', 'APL', 'HEBREW', 'THAI', 'KOREAN';
+    'TECHNICAL', 'SPECIAL', 'PUBLISHING', 'APL', 'HEBREW', 'THAI',  'KOREAN';
 
 # Module to contain all Tk specific functionality
 
-my %windows;    # hash for all window definitions
-my %menus;      # hash for all menu definitions
-my @servers;    # array of servers provided on cmdline
-my %servers;    # hash of server cx info
+my %windows;                   # hash for all window definitions
+my %menus;                     # hash for all menu definitions
+my @servers;                   # array of servers provided on cmdline
+my %servers;                   # hash of server cx info
 my $xdisplay;
 my %keyboardmap;
 my $sysconfigdir = "/etc";
@@ -183,8 +186,8 @@ sub open_client_windows(@) {
         $servers{$server}{realname}       = $realname;
         $servers{$server}{username}       = $self->config->{user};
         $servers{$server}{username}       = $username if ($username);
-        $servers{$server}{username}       = $username || '';
-        $servers{$server}{port}           = $port || '';
+        $servers{$server}{username}       = $username             || '';
+        $servers{$server}{port}           = $port                 || '';
         $servers{$server}{master}         = $self->config->{mstr} || '';
         $servers{$server}{master}         = $master if ($master);
 
@@ -312,6 +315,8 @@ sub set_half_inactive() {
     my ($self) = @_;
     $self->debug( 2, "Setting approx half of all hosts to inactive" );
 
+    return if !%servers;
+
     my (@keys) = keys(%servers);
     $#keys /= 2;
     foreach my $svr (@keys) {
@@ -344,7 +349,8 @@ sub add_host_by_name() {
     if ( $menus{host_entry} ) {
         $self->debug( 2, "host=", $menus{host_entry} );
         my @names
-            = $self->resolve_names( split( /\s+/, $menus{host_entry} ) );
+            = $self->parent->resolve_names(
+            split( /\s+/, $menus{host_entry} ) );
         $self->debug( 0, 'Opening to: ', join( ' ', @names ) ) if (@names);
         $self->open_client_windows(@names);
     }
@@ -352,7 +358,7 @@ sub add_host_by_name() {
     if ( defined $menus{listbox} && $menus{listbox}->curselection() ) {
         my @hosts = $menus{listbox}->get( $menus{listbox}->curselection() );
         $self->debug( 2, "host=", join( ' ', @hosts ) );
-        $self->open_client_windows( $self->resolve_names(@hosts) );
+        $self->open_client_windows( $self->parent->resolve_names(@hosts) );
     }
 
     $self->build_hosts_menu();
@@ -431,8 +437,9 @@ sub load_keyboard_map() {
 
                 # keyboard layout contains the keycode at $modifier level
                 if (defined(
-                        $keyboardmap{ $keycodetosym{ $keyboard[$i][$modifier]
-                        } }
+                        $keyboardmap{
+                            $keycodetosym{ $keyboard[$i][$modifier] }
+                        }
                     )
                     )
                 {
@@ -593,17 +600,20 @@ sub substitute_macros {
         my $macro_servername = $self->config->{macro_servername};
         ( my $servername = $svr ) =~ s/\s+//;
         $text =~ s!$macro_servername!$servername!xsmg;
+        $ENV{CSSH_SERVERNAME} = $servername;
     }
     {
         my $macro_hostname = $self->config->{macro_hostname};
-        my $hostname       = $servers{$svr}{givenname};
+        my $hostname       = $self->config->{hostname_override} || hostname();
         $text =~ s!$macro_hostname!$hostname!xsmg;
+        $ENV{CSSH_HOSTNAME} = $hostname;
     }
     {
         my $macro_username = $self->config->{macro_username};
         my $username       = $servers{$svr}{username};
         $username ||= getpwuid($UID);
         $text =~ s!$macro_username!$username!xsmg;
+        $ENV{CSSH_USERNAME} = $username;
     }
     {
         my $macro_newline = $self->config->{macro_newline};
@@ -613,6 +623,56 @@ sub substitute_macros {
         my $macro_version = $self->config->{macro_version};
         my $version       = $self->parent->VERSION;
         $text =~ s/$macro_version/$version/xsmg;
+        $ENV{CSSH_VERSION} = $version;
+    }
+
+    $ENV{CSSH_CONNECTION_STRING} = $servers{$svr}{connect_string};
+    $ENV{CSSH_CONNECTION_PORT}   = $servers{$svr}{port};
+
+    # Set up environment variables in the macro environment
+    for my $i (qw/ 1 2 3 4 /) {
+        my $macro_user_command = 'macro_user_' . $i . '_command';
+        my $macro_user         = $self->config->{ 'macro_user_' . $i };
+
+        next unless $text =~ $macro_user;
+        if ( !$self->config->{$macro_user_command} ) {
+            $text =~ s/$macro_user//xsmg;
+            next;
+        }
+
+        my $cmd = $self->config->{$macro_user_command};
+
+        local $SIG{CHLD} = undef;
+
+        my $stderr_fh = gensym;
+        my $stdout_fh = gensym;
+        my $child_pid = eval { open3( undef, $stdout_fh, $stderr_fh, $cmd ) };
+
+        if ( my $err = $@ ) {
+
+   # error message is hardcoded into open3 - tidy it up a little for our users
+            $err =~ s/ at .*//;
+            $err =~ s/open3: //;
+            $err =~ s/( failed)/' $1/;
+            $err =~ s/(exec of) /$1 '/;
+            warn "Macro failure for '$macro_user_command': $err";
+            next;
+        }
+        waitpid( $child_pid, 0 );
+        my $cmd_rc = $? >> 8;
+
+        my @stdout = <$stdout_fh>;
+        my @stderr = <$stderr_fh>;
+
+        if ( $cmd_rc > 0 || @stderr ) {
+            warn "Macro failure for '$macro_user_command'", $/;
+            warn "Exited with error output:: @stderr" if @stderr;
+            warn "Exited with non-zero return code: $cmd_rc", $/ if $cmd_rc;
+        }
+        else {
+            #$self->send_text_to_all_servers( $stdout );
+            return join( '', @stdout );
+        }
     }
 
     return $text;
@@ -1149,7 +1209,8 @@ sub setup_repeat() {
 
                         for ($cmd) {
                             if (m/^open$/) {
-                                my @new_hosts = $self->resolve_names(@tags);
+                                my @new_hosts
+                                    = $self->parent->resolve_names(@tags);
                                 $self->open_client_windows(@new_hosts);
                                 $self->build_hosts_menu();
                                 last;
@@ -1220,11 +1281,18 @@ sub setup_repeat() {
 ### Window and menu definitions ###
 
 sub create_windows() {
-    my ($self) = @_;
+    my ($self)          = @_;
+    my ($screen_height) = $xdisplay->{height_in_pixels};
+    my ($screen_width)  = $xdisplay->{width_in_pixels};
     $self->debug( 2, "create_windows: started" );
 
     $windows{main_window}
         = MainWindow->new( -title => "ClusterSSH", -class => 'cssh', );
+    if ( $screen_height * $screen_width >= 8294400 )    # display 4k or bigger
+    {
+        $windows{main_window}->optionAdd( '*font', 'Nimbus 14' )
+            ;    # better for 4k displays
+    }
     $windows{main_window}->withdraw;    # leave withdrawn until needed
 
     if ( defined( $self->config->{console_position} )
@@ -1503,7 +1571,7 @@ sub key_event {
     my $keycode   = $Tk::event->k;
     my $keysymdec = $Tk::event->N;
     my $keysym    = $Tk::event->K;
-    my $state = $Tk::event->s || 0;
+    my $state     = $Tk::event->s || 0;
 
     $menus{entrytext} = "";
 
@@ -1545,6 +1613,18 @@ sub key_event {
                     $self->send_text_to_all_servers(
                         $self->config->{macro_username} )
                         if ( $hotkey eq "key_username" );
+                    $self->send_text_to_all_servers(
+                        $self->config->{macro_user_1} )
+                        if ( $hotkey eq "key_user_1" );
+                    $self->send_text_to_all_servers(
+                        $self->config->{macro_user_2} )
+                        if ( $hotkey eq "key_user_2" );
+                    $self->send_text_to_all_servers(
+                        $self->config->{macro_user_3} )
+                        if ( $hotkey eq "key_user_3" );
+                    $self->send_text_to_all_servers(
+                        $self->config->{macro_user_4} )
+                        if ( $hotkey eq "key_user_4" );
                     $self->add_host_by_name()
                         if ( $hotkey eq "key_addhost" );
                     $self->retile_hosts("force")
@@ -1609,6 +1689,40 @@ sub create_menubar() {
                 -command     => sub { $self->show_history; },
                 -accelerator => $self->config->{key_history},
             ],
+            [   "checkbutton",
+                "Auto Quit",
+                -variable => \$self->config->{auto_quit},
+                -offvalue => 'no',
+                -onvalue  => 'yes',
+            ],
+
+           # While this autoclose menu works as expected, the functionality
+           # within terminals does not.  "auto_close" is set when the terminal
+           # is opened and is not updated when the variable is changed.
+           #
+           #    [   "cascade" => "Auto Close",
+           #        -menuitems => [
+           #            [   "radiobutton",
+           #                "Auto Close",
+           #                -variable    => \$self->config->{auto_close},
+           #                -label    => 'Off',
+           #                -value    => '0',
+           #            ],
+           #            [   "radiobutton",
+           #                "Auto Close",
+           #                -variable    => \$self->config->{auto_close},
+           #                -label    => '5 Seconds',
+           #                -value    => '5',
+           #            ],
+           #            [   "radiobutton",
+           #                "Auto Close",
+           #                -variable    => \$self->config->{auto_close},
+           #                -label    => '10 Seconds',
+           #                -value    => '10',
+           #            ],
+           #        ],
+           #    -tearoff => 0,
+           #    ],
             [   "command",
                 "Exit",
                 -command     => sub { $self->parent->exit_prog },
